/* pstorage_lock.c
Author : Amit Varshney
Date : 29/12/2015
*/

#include <stdint.h>
#include <string.h>
#include "app_timer_appsh.h"
#include "bsp.h"
#include "bsp_btn_ble.h"
#include "nrf_delay.h"
#include "pstorage.h"
#include "pstorage_lock.h"
#include "misc.h"

pstorage_handle_t base_handle;
static bool pstorage_wait_flag = 0;
static pstorage_block_t pstorage_wait_handle = 0;

static void pstorage_cb_handler(pstorage_handle_t  * handle,
															 uint8_t              op_code,
                               uint32_t             result,
                               uint8_t            * p_data,
                               uint32_t             data_len)
{
		if(handle->block_id == pstorage_wait_handle) { pstorage_wait_flag = 0; }  //If we are waiting for this callback, clear the wait flag.
	
		switch(op_code)
		{
			case PSTORAGE_LOAD_OP_CODE:
				 if (result == NRF_SUCCESS)
				 {
						pstorage_wait_flag = 0;						 
						Report("load called\n");	 
						Report("line # %d in pstorage_lock.c\n", __LINE__);
				 }
				 break;
			case PSTORAGE_STORE_OP_CODE:
				 if (result == NRF_SUCCESS)
				 {
					 pstorage_wait_flag = 0;
					 Report("store called\n");
					 Report("line # %d in pstorage_lock.c\n", __LINE__);
				 }
				 break;				 
			case PSTORAGE_UPDATE_OP_CODE:
				 if (result == NRF_SUCCESS)
				 {
					 pstorage_wait_flag = 0;
					 Report("update called\n");
					 Report("line # %d in pstorage_lock.c\n", __LINE__);
				 }
				 break;
			case PSTORAGE_CLEAR_OP_CODE:
				 if (result == NRF_SUCCESS)
				 {
					 pstorage_wait_flag = 0;
					 Report("clear called %d\n", (int)pstorage_wait_flag );
					 Report("line # %d in pstorage_lock.c\n", __LINE__);
				 }
				 break;
			default :
				 break;				 
		}			
}

void pstorage_start()
{
		uint32_t retval;
		pstorage_module_param_t param;
		
		Report("pstorage init\n");
		Report("line # %d in pstorage_lock.c\n", __LINE__);
		retval = pstorage_init();
		if(retval != NRF_SUCCESS)
		{
				bsp_indication_set(BSP_INDICATE_FATAL_ERROR);
		}
		param.block_size = 32;
		param.block_count = 250;
		param.cb = pstorage_cb_handler;   //Set the pstorage callback handler

		Report("pstorage register\n");
		Report("line # %d in pstorage_lock.c\n", __LINE__);
		retval = pstorage_register(&param, &base_handle);
		if (retval != NRF_SUCCESS)
		{
				Report("pstorage_register failed\n");
				Report("line # %d in pstorage_lock.c\n", __LINE__);
				bsp_indication_set(BSP_INDICATE_FATAL_ERROR);
		}
		Report("mode_handle block_id= %d, mode_handle module_id = %d\n", base_handle.block_id, base_handle.module_id);
		Report("line # %d in pstorage_lock.c\n", __LINE__);
}

int pstorage_write(uint8_t *write_data, uint32_t block_offset, uint32_t data_size)
{
		pstorage_handle_t block_handle;
		uint32_t retval, i = 0, j;
		uint8_t data_buffer[uiBlockSize];
		
		//Get block identifiers
		while(i != data_size)
		{
			memset(data_buffer, '\0', uiBlockSize);
			retval = pstorage_block_identifier_get(&base_handle, block_offset, &block_handle);	
			if(retval != NRF_SUCCESS)
			{
				Report("pstorage_block_identifier_get failed\n");
				Report("line # %d in pstorage_lock.c\n", __LINE__);
				bsp_indication_set(BSP_INDICATE_ALERT_0);
				return -1;
			}
			Report("block id = %d, module id = %d, handle block id = %d, handle module id = %d\n",block_handle.block_id,block_handle.module_id,base_handle.block_id, base_handle.module_id);
			Report("line # %d in pstorage_lock.c\n", __LINE__);
		
			pstorage_wait_handle = block_handle.block_id;
			pstorage_wait_flag = 1; 		
			Report("pstorage write \n");
			Report("line # %d in pstorage_lock.c\n", __LINE__);
			
			for(j = 0; ((j < uiBlockSize) && (i < data_size)); i++, j++)
			{
				data_buffer[j] = write_data[i];
			}
			
			retval = pstorage_store(&block_handle, data_buffer, uiBlockSize, 0);				
			if(retval != NRF_SUCCESS)
			{
				Report("pstorage_write failed\n");
				Report("line # %d in pstorage_lock.c\n", __LINE__);
				return -1;
			}
			nrf_delay_ms(100);
			/*while(pstorage_wait_flag == 1)
			{
				nrf_delay_ms(1);
			}*/
			block_offset++;
		}
		return 0;
}

void pstorage_read(unsigned char *read_data, unsigned int block_offset, unsigned int data_size)
{
		/*int i;
		for(i = 0; i < size; i ++)
		{
			read_data[i] = block_offset[i];
		}*/
		uint8_t data_buffer[uiBlockSize];
		pstorage_handle_t block_handle;
		uint32_t retval, i = 0, j;
	
		while(i != data_size)
		{
			//Get block identifiers
			retval = pstorage_block_identifier_get(&base_handle, block_offset, &block_handle);	
			if(retval != NRF_SUCCESS)
			{
				Report("pstorage_block_identifier_get failed\n");
				bsp_indication_set(BSP_INDICATE_ALERT_0);
			}
			Report("block id = %d, module id = %d, handle block id = %d, handle module id = %d\n",block_handle.block_id,block_handle.module_id,base_handle.block_id, base_handle.module_id);
		
			pstorage_wait_flag = 1; 		
			Report("pstorage load \n");
			retval = pstorage_load(data_buffer, &block_handle, uiBlockSize, 0);				
			if(retval != NRF_SUCCESS)
			{
				Report("pstorage_load failed\n");
				bsp_indication_set(BSP_INDICATE_RCV_ERROR);
			}
			nrf_delay_ms(100);

			/*while(pstorage_wait_flag == 1)
			{
				nrf_delay_ms(1);
			}*/
			for(j = 0; ((j < uiBlockSize) && (i < data_size)); i++, j++)
			{
				read_data[i] = data_buffer[j];
			}
			block_offset++;
		}
}

int pstorage_erase(unsigned int block_offset, unsigned int data_size)
{
		pstorage_handle_t block_handle;
		uint32_t retval;
		
		//Get block identifiers
		retval = pstorage_block_identifier_get(&base_handle, block_offset, &block_handle);	
		if(retval != NRF_SUCCESS)
		{
				Report("pstorage_block_identifier_get failed\n");
				Report("line # %d in pstorage_lock.c\n", __LINE__);
				return -1;
		}
		Report("block id = %d, module id = %d, handle block id = %d, handle module id = %d\n",block_handle.block_id,block_handle.module_id,base_handle.block_id, base_handle.module_id);
		Report("line # %d in pstorage_lock.c\n", __LINE__);
		
		//pstorage_wait_handle = block_handle.block_id;		
		pstorage_wait_flag = 1; 		
		Report("pstorage erase \n");
		Report("line # %d in pstorage_lock.c\n", __LINE__);
		retval = pstorage_clear(&block_handle, data_size);				
		if(retval != NRF_SUCCESS)
		{
				Report("pstorage_erase failed\n");
				Report("line # %d in pstorage_lock.c\n", __LINE__);
				bsp_indication_set(BSP_INDICATE_RCV_ERROR);
				return -1;
		}
		nrf_delay_ms(100);

		/*while(pstorage_wait_flag == 1)
		{
			nrf_delay_ms(1);
		}*/
		return 0;
}
